import os
import pickle
from openpyxl import Workbook, load_workbook
from openpyxl.utils.dataframe import dataframe_to_rows
from datetime import datetime
import pandas as pd
import logging
import pickle
from pathlib import Path

class FileManager:
    def __init__(self):
        logging.basicConfig(level=logging.INFO)
        self.logger = logging.getLogger(__name__)

    @staticmethod
    def make_dir(folder_path):
        relative_path = folder_path.split("/")
        current_path = os.getcwd()
        for folder_name in relative_path:
            current_path = os.path.join(current_path, folder_name)
            if not os.path.exists(current_path):
                os.makedirs(current_path)
                logging.info(f"폴더 '{folder_name}'가 생성되었습니다.")

    @staticmethod
    def delete_dir(folder_path):
        try:
            if os.path.exists(folder_path):
                for root, dirs, files in os.walk(folder_path, topdown=False):
                    for name in files:
                        file_path = os.path.join(root, name)
                        os.remove(file_path)
                        logging.info(f"파일 '{file_path}'가 삭제되었습니다.")
                    for name in dirs:
                        dir_path = os.path.join(root, name)
                        os.rmdir(dir_path)
                        logging.info(f"디렉토리 '{dir_path}'가 삭제되었습니다.")
                os.rmdir(folder_path)
                logging.info(f"The folder at {folder_path} has been deleted.")
            else:
                logging.warning(f"The folder at {folder_path} does not exist.")
        except Exception as e:
            logging.error(f"Error: {e}")

    @staticmethod
    def load_from_pickle(storage_name, data_path: Path = Path.cwd()):
        with open(f"{data_path}/{storage_name}.pickle", 'rb') as f:
            return pickle.load(f)
        
    @staticmethod
    def save_to_pickle(data, storage_name, data_path: Path = Path.cwd()):
        with open(f"{data_path}/{storage_name}.pickle", 'wb') as f:
            pickle.dump(data, f)

    @staticmethod
    def get_datetime_info(include_time=True):
        format_str = "%Y-%m-%d_%H-%M-%S" if include_time else "%Y-%m-%d"
        return datetime.now().strftime(format_str)

    @staticmethod
    def get_name_from_url(url):
        if url[-1] == '/':
            url = url[:-1]
        name = url.rsplit('/', 1)[-1]
        logging.info(f"URL '{url}'에서 이름 '{name}'이(가) 추출되었습니다.")
        return name

    @staticmethod
    def dict_to_excel(dict_data, file_name="dictToExcel", sheet_name="sheet1", orient_idx=True):
        orient = 'columns'
        if orient_idx: orient = 'index'

        df = pd.DataFrame.from_dict(dict_data, orient=orient).reset_index().rename(columns={'index': 'model'})

        try:
            wb = load_workbook(filename=f"{file_name}.xlsx")
            ws = wb.create_sheet(title=sheet_name)
            logging.info(f"기존 엑셀 파일 '{file_name}.xlsx'에서 시트 '{sheet_name}'이(가) 생성되었습니다.")
        except FileNotFoundError:
            wb = Workbook()
            ws = wb.active
            ws.title = sheet_name
            logging.info(f"새 엑셀 파일 '{file_name}.xlsx'가 생성되었습니다.")

        for row in dataframe_to_rows(df, index=False, header=True):
            ws.append(row)

        wb.save(f"{file_name}.xlsx")
        logging.info(f"데이터가 {file_name}.xlsx 파일의 {sheet_name} 시트에 저장되었습니다.")

    @staticmethod
    def df_to_excel(df_data, file_name="dfToExcel", sheet_name="sheet1", mode='w'):
        # Create Excel writer
        with pd.ExcelWriter(file_name, engine='openpyxl', mode=mode) as writer:
            # Write each DataFrame to a different sheet
            df_data.to_excel(writer, sheet_name=sheet_name, index=False)
            logging.info(f"DataFrame이 {file_name}.xlsx 파일의 {sheet_name} 시트에 저장되었습니다.")



   


from google.cloud import firestore
from google.oauth2 import service_account
from datetime import datetime
import logging
import pandas as pd

class FirestoreManager:
    def __init__(self, credentials_file=None):
        # 로깅 설정
        logging.basicConfig(level=logging.INFO)
        
        if credentials_file is not None:
            self.credentials = service_account.Credentials.from_service_account_file(credentials_file)
            logging.info("Credentials loaded successfully.")
            self.db = firestore.Client(credentials=self.credentials)
        else:
            self.db = firestore.Client()
        
        logging.debug("Firestore client initialized successfully.")
        
    def save_dataframe(self, dict_data: dict, collection_name=None):
        collection_doc = self.db.collection(collection_name) if collection_name else self.db.collection("default_collection")

        for data_name, dataframe in dict_data.items():
            # 데이터프레임을 컬렉션의 문서로 저장
            for index, row in dataframe.iterrows():
                doc_key = f'{data_name}_doc_{index}'  # 문서 키 생성
                doc_ref = collection_doc.document(doc_key)  # 컬렉션 문서 참조
                doc_ref.set(row.to_dict())  # 데이터 저장
                logging.debug(f"Document {doc_key} added successfully under {collection_name}.")
        
        logging.info(f"Data saved successfully under collection {collection_name if collection_name else 'default_collection'}.")




    def read_data(self, collection_name):
        logging.info(f"Reading data from collection: {collection_name}.")
        docs = self.db.collection(collection_name).stream()
        data = []
        
        for doc in docs:
            data.append(doc.to_dict())
            logging.debug(f"Document {doc.id} retrieved successfully from {collection_name}.")
        
        if data:
            df = pd.DataFrame(data)
            logging.info(f"Successfully converted {len(data)} documents to DataFrame.")
            return df
        else:
            logging.warning(f"No documents found in collection: {collection_name}.")
            return pd.DataFrame() 



    # def save_list(self, dict_list: dict, collection_name="sample_collection"):
    #     """
    #     dict_list = {doc_key: data_list}
    #     """
    #     for doc_key, data_list in dict_list.items():
    #         # Firestore 컬렉션 및 문서 경로 지정
    #         doc_ref = self.db.collection(collection_name).document(doc_key)
    #         doc_ref.set({'data_list': data_list})
    
    def save_list(self, dict_list: dict, collection_name="sample_collection"):
        """
        dict_list = {doc_key: data_list}
        """
        for doc_key, data_list in dict_list.items():
            # Firestore 컬렉션 및 문서 경로 지정
            logging.info(f"Saving document with key: {doc_key}")  # 디버그 출력
            doc_ref = self.db.collection(collection_name).document(doc_key)
            print(f"Document reference path: {doc_ref.path}")  # 디버그 출력
            try:
                doc_ref.set({'data_list': data_list})
            except Exception as e:
                print(f"Error saving document: {e}")  # 오류 메시지 출력

    def read_list(self, doc_key, collection_name="sample_collection") -> list:
        """
        Firestore에서 리스트 데이터를 읽어옵니다.
        """
        doc_ref = self.db.collection(collection_name).document(doc_key)
        doc = doc_ref.get()
        
        if doc.exists:
            return doc.to_dict().get('data_list')
        else:
            return []



from selenium import webdriver
from selenium.webdriver.common.action_chains import ActionChains
from selenium.webdriver.common.keys import Keys
from selenium.webdriver.chrome.service import Service
from selenium.webdriver.chrome.options import Options
import time
import os 
import logging
from tools.web.installer import Installer

class WebDriver:
    def __init__(self, headless=False):
        self.SELENIUM_URL = os.getenv('SELENIUM_URL')
        self.headless= headless
        self.driver = None
        if self.SELENIUM_URL is not None: logging.info("Using remote WebDriver.")
        self._init_chrome()

    def _init_chrome(self):
        logging.debug("Initializing Chrome WebDriver.")
        self.chrome_options = Options()
        self.chrome_options.add_argument(
            "--user-agent='Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/99.0.4844.84 Safari/537.36'"
        )  

        if self.headless:
            self.chrome_options.add_argument('--headless=new')  # 헤드리스 모드로 실행
        self.chrome_options.add_argument('--no-sandbox')  # 사용자 네임스페이스 옵션 없이 실행
        self.chrome_options.add_argument('--disable-dev-shm-usage')
        self.chrome_options.add_argument("--window-size=1920,1080")

        if self.SELENIUM_URL is None:
            logging.debug("Installing Chrome and ChromeDriver.")
            webdriver_path = Installer.install_chrome_and_driver()
            self.webdriver_path = webdriver_path.get('driver_path')
            self.browser_path = webdriver_path.get('chrome_path')
            logging.debug(f"ChromeDriver path: {self.webdriver_path}")
            logging.debug(f"Chrome browser path: {self.browser_path}")
            self.chrome_options.binary_location = self.browser_path
        logging.debug("Chrome WebDriver initialized successfully.")
        return None
    
    def get_chrome(self):
        if self.SELENIUM_URL is not None:
            self.driver = webdriver.Remote(
                command_executor=self.SELENIUM_URL,
                options=self.chrome_options
            )
        else:
            self.chrome_options.binary_location = self.browser_path
            service = Service(executable_path=self.webdriver_path)  
            self.driver = webdriver.Chrome(service=service, options=self.chrome_options)
        return self.driver
    
    def move_element_to_center(self, element):
        """ 주어진 웹 요소를 뷰포트의 중앙으로 이동시킵니다. """
        logging.debug("Moving element to center.")
        driver = self.driver
        ActionChains(driver).move_to_element(element).perform()

        # 뷰포트 크기 얻기
        viewport_width = driver.execute_script("return window.innerWidth;")
        viewport_height = driver.execute_script("return window.innerHeight;")

        # 객체의 위치 얻기
        object_x = element.location['x']
        object_y = element.location['y']
        object_width = element.size['width']
        object_height = element.size['height']

        # 화면 중앙으로 이동
        target_x = object_x + object_width / 2 - viewport_width / 2
        target_y = object_y + object_height / 2 - viewport_height / 2
        driver.execute_script(f"window.scrollTo({target_x}, {target_y});")
        logging.debug(f"Element moved to center at ({target_x}, {target_y}).")
        return None

    def getDistanceScrollToBtm(self):
        """ 스크롤을 페이지 아래로 내립니다. """
        logging.debug("Scrolling to the bottom of the page.")
        driver = self.driver
        driver.find_element_by_tag_name('body').send_keys(Keys.END)
        time.sleep(1)  # 스크롤이 내려가는 동안 대기

        # 스크롤이 이동한 거리 계산
        scrollDistance = driver.execute_script("return window.pageYOffset;")
        logging.debug(f"Scrolled distance: {scrollDistance} pixels.")

        # Home 키를 눌러 시작 위치로 복귀
        driver.find_element_by_tag_name('body').send_keys(Keys.HOME)
        time.sleep(1)  # 스크롤이 위로 올라가는 동안 대기
        return scrollDistance

    def get_scroll_distance_total(self):
        """ 페이지 스크롤의 총 거리를 계산합니다. """
        logging.debug("Calculating total scroll distance.")
        total_scroll_distance = 0
        prev_scroll_distance = -1
        
        while True:
            scroll_distance = self.__get_scroll_distance__()
            total_scroll_distance += scroll_distance
            if scroll_distance == 0 or scroll_distance == prev_scroll_distance:
                break
            prev_scroll_distance = scroll_distance
        
        # 초기 위치로 복귀하기 위해 페이지 맨 위로 스크롤합니다.
        self.driver.execute_script("window.scrollTo(0, 0)")
        time.sleep(1)
        logging.debug(f"Total scroll distance: {total_scroll_distance} pixels.")
        return total_scroll_distance

    def __get_scroll_distance__(self):
        driver = self.driver
        # 현재 페이지의 스크롤 위치를 가져옵니다.
        current_scroll_position = driver.execute_script("return window.scrollY || window.pageYOffset")
        logging.debug(f"Current scroll position: {current_scroll_position} pixels.")
        
        # 스크롤 이벤트를 발생시킵니다.
        driver.execute_script("window.scrollTo(0, document.body.scrollHeight)")
        time.sleep(1)
        
        # 새로운 스크롤 위치를 가져옵니다.
        new_scroll_position = driver.execute_script("return window.pageYOffset")
        # 페이지 이동 거리를 계산합니다.
        scroll_distance = new_scroll_position - current_scroll_position
        logging.debug(f"Scrolled distance: {scroll_distance} pixels.")
        return scroll_distance



from setuptools import setup, find_packages

setup(
    name='xik_tools',
    version='0.61',
    author='xikest',
    description='tools for web',
    packages=find_packages(),
    python_requires='>=3.8',
    install_requires=[
        'pandas',
        'selenium','openpyxl'
    ],

    entry_points={
        'console_scripts': [
            'install_env_script = my_package.install_env:main',
        ],
    },

    url='https://github.com/xikest/tools',  # GitHub 프로젝트 페이지 링크
    project_urls={
        'Source': 'https://github.com/xikest/tools',  # GitHub 프로젝트 페이지 링크
        'Bug Tracker': 'https://github.com/xikest/tools',  # 버그 트래커 링크
    },
)



import requests
import base64
import pandas as pd
import json

class GitMgt:
    
    def __init__(self):
        pass
    
    @staticmethod    
    def upload_json_to_github(content, git_token, git_repo, file_name='data'):
        
        if isinstance(content, pd.DataFrame):
            json_content = content.to_json(orient='records', lines=True)
        elif isinstance(content, dict):
            json_content = json.dumps(content)
        else:
            raise ValueError("content는 DataFrame, dict 이어야 합니다.")
        
        encoded_content = base64.b64encode(json_content.encode('utf-8')).decode('utf-8')
        
        url = f'https://api.github.com/repos/{git_repo}/contents/{file_name}.json'
        
        headers = {
            'Authorization': f'token {git_token}',
            'Accept': 'application/vnd.github.v3+json'
        }
        
        data = {
            'message': 'Upload JSON file',
            'content': encoded_content, 
        }

        response = requests.put(url, headers=headers, json=data)

        if response.status_code == 201:
            print('파일이 성공적으로 업로드되었습니다!')
        else:
            print('업로드 실패:', response.json())
            
    def get_github_folder_files(owner, repo, path, branch="main"):
        url = f"https://api.github.com/repos/{owner}/{repo}/contents/{path}?ref={branch}"
        headers = {"Accept": "application/vnd.github.v3+json"}
        response = requests.get(url, headers=headers)

        if response.status_code == 200:
            files = response.json()
            # 각 파일에 접근할 수 있는 raw URL을 생성하여 반환
            return [f"https://raw.githubusercontent.com/{owner}/{repo}/{branch}/{file['path']}" for file in files if file['type'] == 'file']
        else:
            print(f"Error {response.status_code}: {response.json().get('message')}")
            return []